import socketio
import time as time

sio = socketio.Client()

# try to connect
try:
    sio.connect('http://localhost:3000')
except Exception as e:
    print(e)
    pass

# Global vars to remember the results by
RESULTS = {}
REQ_IDS = []
FINISHED = None
DONE = False
RESULT = None


# When we get a result message in particular
@sio.on('result')
def on_result(data):
    global RESULTS, REQ_IDS, FINISHED, RESULT, DONE
    
    # Get this request ID
    req_id = data.get('request',{}).get('id')
    new_word = data.get('response','')

    # And if it's not already in results
    if not req_id in RESULTS:
        # then initially stuff it with this data
        RESULTS[req_id] = data
        # and add this request id to the last 
        REQ_IDS.append(req_id)
    # If it's already in results
    else:
        # then simply add the new response word
        RESULTS[req_id]['response'] += new_word    

    FINISHED = str(new_word).strip()
    if FINISHED == "<end>" or FINISHED == "\n":
        DONE = True
        if REQ_IDS and RESULTS:
            # get latest id
            req_id = REQ_IDS[-1]
            # get result dictionary from latest id as key
            result = RESULTS[req_id]
            # return result
            RESULT = result

# Emit the request
def generate(request):
    sio.emit('request', request)
    while not DONE:
        time.sleep(0.01)
    return RESULT







import internal_funcs as funcs
import sys

class Dalai:
    def generate_request(self, prompt, model, debug=False, id='TS-1679621517384-9980', n_predict=25, repeat_last_n=64, repeat_penalty=1.3, seed=-1, temp=0.5, threads=4, top_k=40, top_p=0.9):
        request = {'debug': False, 'id':id, 'model':model, 'models':[model], 'n_predict':n_predict, 'prompt':prompt, 'repeat_last_n':repeat_last_n, 'repeat_penalty':repeat_penalty, 'seed':seed, 'temp':temp, 'threads':threads, 'top_k':top_k, 'top_p':top_p}
        return request
    
    def generate(self, prompt, prettify=True):
        if prettify == False:
            return funcs.generate(prompt)

        else:
            response = funcs.generate(prompt)['response']
            response = response.replace("\n", "")
            response = response.replace("\r", "")
            response = response.replace("<end>", "")
            if not response.endswith(".") :
                response += "."
            return response

